# Generated by Django 2.2.5 on 2019-09-11 23:45

from django.db import migrations
from aristotle_mdr.contrib.custom_fields.utils import get_system_name, generate_random_unique_characters


def generate_system_name(apps, schema_editor):
    """ Generate the registry-wide unique system name """
    CustomField = apps.get_model('aristotle_mdr_custom_fields', 'CustomField')

    for custom_field in CustomField.objects.all():
        if not custom_field.system_name:
            # If a system name has not been created
            system_name = get_system_name(custom_field.allowed_model, custom_field.name)

            while CustomField.objects.filter(system_name=system_name).count() != 0:
                # Ensure that the generated system name is unique
                system_name = '{system_name}_{random_chars}'.format(system_name=system_name,
                                                                    random_chars=generate_random_unique_characters())
            custom_field.system_name = system_name

            custom_field.save()


def remove_system_name(apps, schema_editor):
    CustomField = apps.get_model('aristotle_mdr_custom_fields', 'CustomField')

    for custom_field in CustomField.objects.all():
        custom_field.system_name = ''
        custom_field.save()


class Migration(migrations.Migration):
    dependencies = [
        ('aristotle_mdr_custom_fields', '0010_customfield_system_name'),
    ]

    operations = [
        migrations.RunPython(generate_system_name, remove_system_name),
    ]
